package File::CounterFile::Date;
# -*- Perl -*- Wed Nov 12 10:08:14 CST 2003 
############################################################################### 
# Written by Tim Skirvin <tskirvin@killfile.org>
# Copyright 2003, Tim Skirvin.  Redistribution terms are below.
###############################################################################
use vars qw( $VERSION @ISA );
$VERSION = "1.0";
@ISA = "File::CounterFile";

=head1 NAME

File::CounterFile::Date - Persistent counter class with creation date

=head1 SYNOPSIS

  use File::CounterFile::Date;
  $c = File::CounterFile::Date->new("COUNTER", "aa00");

  $c->lock;
  $c++;
  my $value = $c->value || 0;  
  my $date = $c->created || time;
  $c->unlock;

=head1 DESCRIPTION

This library is a superset of File::CounterFile, which keeps track of the
date at which the given entry was created.  This is mostly useful for web
logs, but may be useful for something else.

I won't repeat most of the documentation here.  Check on the manpages for
C<File::CounterFile>.

=cut

###############################################################################
### main() ####################################################################
###############################################################################

use Carp   qw(croak);
use File::CounterFile;
use Fcntl qw(O_RDWR O_CREAT);

use strict;

# These variables are really not meant for use configuration
use vars qw( $MAGIC $DEFAULT_INITIAL $DEFAULT_DIR);

$MAGIC = "#COUNTER-DATE-1.0\n";        # first line in counter files
$DEFAULT_INITIAL = 0;                  # default initial counter value

 # default location for counter files
$DEFAULT_DIR = $ENV{TMPDIR} || "/usr/tmp";

=head2 Updated Functions 

=over 4

=item new ( FILE [, INITIAL] )

Creates a counter file at C<FILE>.  Very similar to what's actually in
File::CounterFile, but we have to override it so we can read and write the
date information as well.

=cut

sub new
{
    my($class, $file, $initial) = @_;
    croak("No file specified\n") unless defined $file;

    $file = "$DEFAULT_DIR/$file" unless $file =~ /^[\.\/]/;
    $initial = $DEFAULT_INITIAL unless defined $initial;

    my $value;
    local($/, $\) = ("\n", undef);
    local *F;
    sysopen(F, $file, O_RDWR|O_CREAT) or croak("Can't open $file: $!");
    flock(F, 2) or croak("Can't flock: $!");
    my $first_line = <F>;  # chomp $first_line;
    my $date = time;
    if (defined $first_line) {
	croak "Bad counter magic '$first_line' in $file (expecting '$MAGIC')" unless $first_line eq $MAGIC;
	my $nextline = <F>;  chomp $nextline;
	($value, $date) = split (/\s+/, $nextline);
	$date ||= 0;  $value ||= 0; 
    }
    else {
	seek(F, 0, 0);
	print F "$MAGIC";
	print F "$initial $date\n";
	$value = $initial;
    }
    close(F) || croak("Can't close $file: $!");

    bless { file    => $file,  # the filename for the counter
	   'value'  => $value, # the current value
	   'create' => $date,  # the date of creation
	    updated => 0,      # flag indicating if value has changed
	    # handle => XXX,   # file handle symbol. Only present when locked
	  };
}

=item lock ()

Locks the filehandle, so we can get information in and out without having
to always write the file out every time.  Mostly uses the code from
File::CounterFile::lock(), but has to work around the need to use the
different MAGIC string.

Note that we aren't using SUPER, against my better judgement, because it
didn't seem to want to work perfectly.

=cut

sub lock {
  my $orig = $File::CounterFile::MAGIC;
  $File::CounterFile::MAGIC = $MAGIC;
  my @return = File::CounterFile::lock(@_);
  $File::CounterFile::MAGIC = $orig;
  @return;
}

=item unlock ()

Writes out the counter information and unlocks the filehandle.  Had to be
updated so we could write the creation time out, as well as the updated
version string.

=cut

sub unlock {
    my($self) = @_;
    return unless $self->locked;

    my $fh = $self->{handle};

    if ($self->{updated}) {
	# write back new value
	local($\) = undef;
	seek($fh, 0, 0) or croak "Can't seek to beginning: $!";
	print $fh "$MAGIC";
	print $fh "$self->{'value'} $self->{'create'}\n";
    }

    close($fh) or warn "Can't close: $!";
    delete $self->{handle};
    $self;
}

=item Version ()

Returns the version number.  Had to be included again so we'd get the
version number of this module, and not File::CounterFile.

=cut

sub Version { $VERSION }

=back

=head2 New Functions 

=over 4

=item created ()

Returns the time that the counter file was created, in seconds since the
epoch (Unix date format).  This can be later formatted however you'd like.

=cut

sub created {
    my ($self) = @_;
    my $date;
    if ($self->locked) { $date = $self->{'create'}; }
    else  { $self->lock; $date = $self->{'create'}; $self->unlock; }
    $date;
}

=back

=head1 REQUIREMENTS

News::CounterFile 1.01 

=head1 SEE ALSO

B<News::CounterFile>

=head1 NOTES

This was designed to work with File::CounterFile 1.01; I don't know how
well it'll work with other versions.  Probably not at all.  

=head1 AUTHOR

Tim Skirvin <tskirvin@killfile.org>, based on work by Gisle Aas.

=head1 COPYRIGHT

Copyright 2003 by Tim Skirvin <tskirvin@killfile.org>.  This code may be
distributed under the same terms as Perl itself.

Based on code by Gisle Aas <gisle@aas.no>, which was under the same
license.

=cut

###############################################################################
### Version History ###########################################################
###############################################################################
# v1.0		Wed Nov 12 10:08:14 CST 2003 
### Commented and ready for distribution.
